@@ -2,6 +2,8 @@
  *
  * Copyright (C) 2010 Collabora, Ltd.
  *
+ * SPDX-License-Identifier: LGPL-2.1-or-later
+ *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
@@ -67,6 +69,8 @@ g_proxy_resolver_default_init (GProxyResolverInterface *iface)
 {
 }
 
+static GProxyResolver *proxy_resolver_default_singleton = NULL;  /* (owned) (atomic) */
+
 /**
  * g_proxy_resolver_get_default:
  *
@@ -80,9 +84,18 @@ g_proxy_resolver_default_init (GProxyResolverInterface *iface)
 GProxyResolver *
 g_proxy_resolver_get_default (void)
 {
-  return _g_io_module_get_default (G_PROXY_RESOLVER_EXTENSION_POINT_NAME,
-				   "GIO_USE_PROXY_RESOLVER",
-				   (GIOModuleVerifyFunc)g_proxy_resolver_is_supported);
+  if (g_once_init_enter (&proxy_resolver_default_singleton))
+    {
+      GProxyResolver *singleton;
+
+      singleton = _g_io_module_get_default (G_PROXY_RESOLVER_EXTENSION_POINT_NAME,
+                                            "GIO_USE_PROXY_RESOLVER",
+                                            (GIOModuleVerifyFunc) g_proxy_resolver_is_supported);
+
+      g_once_init_leave (&proxy_resolver_default_singleton, singleton);
+    }
+
+  return proxy_resolver_default_singleton;
 }
 
 /**
