@@ -87,6 +87,39 @@ test_atomic_int_compare_and_exchange (void)
 #endif
 }
 
+static void
+test_atomic_pointer_exchange (void)
+{
+#if __cplusplus >= 201103L
+  const gchar *str1 = "str1";
+  const gchar *str2 = "str2";
+  const gchar *atomic_string = str1;
+
+  g_test_message ("Test that g_atomic_pointer_exchange() with a "
+                  "non-void* pointer doesn’t have any compiler warnings in C++ mode");
+
+  g_assert_true (g_atomic_pointer_exchange (&atomic_string, str2) == str1);
+  g_assert_true (atomic_string == str2);
+#else
+  g_test_skip ("This test requires a C++11 compiler");
+#endif
+}
+
+static void
+test_atomic_int_exchange (void)
+{
+#if __cplusplus >= 201103L
+  gint atomic_int = 5;
+
+  g_test_message ("Test that g_atomic_int_compare_and_exchange() doesn’t have "
+                  "any compiler warnings in C++ mode");
+
+  g_assert_cmpint (g_atomic_int_exchange (&atomic_int, 50), ==, 5);
+#else
+  g_test_skip ("This test requires a C++11 compiler");
+#endif
+}
+
 int
 main (int argc, char *argv[])
 {
@@ -99,6 +132,8 @@ main (int argc, char *argv[])
   g_test_add_func ("/C++/typeof", test_typeof);
   g_test_add_func ("/C++/atomic-pointer-compare-and-exchange", test_atomic_pointer_compare_and_exchange);
   g_test_add_func ("/C++/atomic-int-compare-and-exchange", test_atomic_int_compare_and_exchange);
+  g_test_add_func ("/C++/atomic-pointer-exchange", test_atomic_pointer_exchange);
+  g_test_add_func ("/C++/atomic-int-exchange", test_atomic_int_exchange);
 
   return g_test_run ();
 }
