@@ -4,7 +4,7 @@
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
- * version 2 of the License, or (at your option) any later version.
+ * version 2.1 of the License, or (at your option) any later version.
  *
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -32,21 +32,12 @@ static FILE *f_out = NULL;
 static GType root = 0;
 static gboolean recursion = TRUE;
 
-#if 0
-#  define	O_SPACE	"\\as"
-#  define	O_ESPACE " "
-#  define	O_BRANCH "\\aE"
-#  define	O_VLINE "\\al"
-#  define	O_LLEAF	"\\aL"
-#  define	O_KEY_FILL "_"
-#else
-#  define	O_SPACE	" "
-#  define	O_ESPACE ""
-#  define	O_BRANCH "+"
-#  define	O_VLINE "|"
-#  define	O_LLEAF	"`"
-#  define	O_KEY_FILL "_"
-#endif
+#define	O_SPACE	" "
+#define	O_ESPACE ""
+#define	O_BRANCH "├"
+#define	O_VLINE "│"
+#define	O_LLEAF	"└"
+#define	O_KEY_FILL "_"
 
 static void
 show_nodes (GType        type,
@@ -61,10 +52,6 @@ show_nodes (GType        type,
   
   children = g_type_children (type, NULL);
   
-  if (type != root)
-    for (i = 0; i < spacing; i++)
-      g_fprintf (f_out, "%s%s\n", indent, O_VLINE);
-  
   g_fprintf (f_out, "%s%s%s%s",
 	   indent,
 	   sibling ? O_BRANCH : (type != root ? O_LLEAF : O_SPACE),
@@ -96,18 +83,18 @@ show_nodes (GType        type,
 }
 
 static gint
-help (gchar *arg)
+help (const gchar *arg)
 {
-  g_fprintf (stderr, "usage: gobject-query <qualifier> [-r <type>] [-{i|b} \"\"] [-s #] [-{h|x|y}]\n");
-  g_fprintf (stderr, "       -r       specifiy root type\n");
-  g_fprintf (stderr, "       -n       don't descend type tree\n");
-  g_fprintf (stderr, "       -h       guess what ;)\n");
-  g_fprintf (stderr, "       -b       specify indent string\n");
-  g_fprintf (stderr, "       -i       specify incremental indent string\n");
-  g_fprintf (stderr, "       -s       specify line spacing\n");
-  g_fprintf (stderr, "qualifiers:\n");
-  g_fprintf (stderr, "       froots   iterate over fundamental roots\n");
-  g_fprintf (stderr, "       tree     print type tree\n");
+  g_fprintf (stdout, "usage: gobject-query <qualifier> [-r <type>] [-{i|b} \"\"] [-s #] [-{h|x|y}]\n");
+  g_fprintf (stdout, "       -r       specify root type\n");
+  g_fprintf (stdout, "       -n       don't descend type tree\n");
+  g_fprintf (stdout, "       -h       show help\n");
+  g_fprintf (stdout, "       -b       specify indent string\n");
+  g_fprintf (stdout, "       -i       specify incremental indent string\n");
+  g_fprintf (stdout, "       -s       specify line spacing\n");
+  g_fprintf (stdout, "qualifiers:\n");
+  g_fprintf (stdout, "       froots   iterate over fundamental roots\n");
+  g_fprintf (stdout, "       tree     print type tree\n");
   
   return arg != NULL;
 }
@@ -120,7 +107,7 @@ main (gint   argc,
   gboolean gen_froots = 0;
   gboolean gen_tree = 0;
   gint i;
-  gchar *iindent = "";
+  const gchar *iindent = "";
 
   f_out = stdout;
   
@@ -183,11 +170,13 @@ main (gint   argc,
 	{
 	  gen_tree = 1;
 	}
-      else if (strcmp ("-h", argv[i]) == 0)
-	{
-	  return help (NULL);
-	}
-      else if (strcmp ("--help", argv[i]) == 0)
+      else if (strcmp ("--version", argv[i]) == 0)
+        {
+          g_print (PACKAGE_VERSION "\n");
+          return 0;
+        }
+      else if (strcmp ("-h", argv[i]) == 0 ||
+               strcmp ("--help", argv[i]) == 0)
 	{
 	  return help (NULL);
 	}
@@ -196,7 +185,7 @@ main (gint   argc,
     }
   
   if (!gen_froots && !gen_tree)
-    return help (argv[i-1]);
+    return help ((argc > 0) ? argv[i-1] : NULL);
   
   if (!indent_inc)
     {
@@ -213,9 +202,13 @@ main (gint   argc,
       for (i = 0; i <= G_TYPE_FUNDAMENTAL_MAX; i += G_TYPE_MAKE_FUNDAMENTAL (1))
 	{
 	  const gchar *name = g_type_name (i);
+          GType sibling = i + G_TYPE_MAKE_FUNDAMENTAL (1);
+
+          if (sibling > G_TYPE_FUNDAMENTAL_MAX || g_type_name (sibling) == NULL)
+            sibling = 0;
 	  
 	  if (name)
-	    show_nodes (i, 0, iindent);
+	    show_nodes (i, sibling, iindent);
 	}
     }
   
