@@ -20,13 +20,21 @@
 
 """Integration tests for gobject-query utility."""
 
+import collections
+import os
+import shutil
+import subprocess
+import sys
+from textwrap import dedent
 import unittest
 
 import taptestrunner
-import testprogramrunner
 
 
-class TestGobjectQuery(testprogramrunner.TestProgramRunner):
+Result = collections.namedtuple("Result", ("info", "out", "err"))
+
+
+class TestGobjectQuery(unittest.TestCase):
     """Integration test for running gobject-query.
 
     This can be run when installed or uninstalled. When uninstalled, it
@@ -36,10 +44,48 @@ class TestGobjectQuery(testprogramrunner.TestProgramRunner):
     handling of command line arguments, and its exit statuses.
     """
 
-    PROGRAM_NAME = "gobject-query"
+    def setUp(self):
+        self.timeout_seconds = 10  # seconds per test
+        if "G_TEST_BUILDDIR" in os.environ:
+            self.__gobject_query = os.path.join(
+                os.environ["G_TEST_BUILDDIR"], "..", "gobject-query"
+            )
+        else:
+            self.__gobject_query = shutil.which("gobject-query")
+        print("gobject-query:", self.__gobject_query)
 
     def runGobjectQuery(self, *args):
-        return self.runTestProgram(args)
+        argv = [self.__gobject_query]
+
+        # shebang lines are not supported on native
+        # Windows consoles
+        if os.name == "nt":
+            argv.insert(0, sys.executable)
+
+        argv.extend(args)
+        print("Running:", argv)
+
+        env = os.environ.copy()
+        env["LC_ALL"] = "C.UTF-8"
+        print("Environment:", env)
+
+        # We want to ensure consistent line endings...
+        info = subprocess.run(
+            argv,
+            timeout=self.timeout_seconds,
+            stdout=subprocess.PIPE,
+            stderr=subprocess.PIPE,
+            env=env,
+            universal_newlines=True,
+        )
+        info.check_returncode()
+        out = info.stdout.strip()
+        err = info.stderr.strip()
+
+        result = Result(info, out, err)
+
+        print("Output:", result.out)
+        return result
 
     def test_help(self):
         """Test the --help argument."""
